define([
    "./togglebutton.builder",
    "jquery",
    "css!./togglebutton.css",
    "css!./playmodes.css"
],
function(
    ToggleButtonBuilder,
    jq
){

    "use strict";

    /**
     * @constructor
     */

     return function( options, context, metadata ){

        var playmodesButton,
            button,
            playmodesButtonHotSpotElement,
            playmodesContainer;
        
        //HELPER FUNCTION
        function hexToRgb(hex) {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            return result ? {
                r: parseInt(result[1], 16),
                g: parseInt(result[2], 16),
                b: parseInt(result[3], 16)
            } : {
                r: 0,
                g: 0,
                b: 0
            };
        }

        function asRgbaValue( colorValue, transparencyValue ) {

            var color = hexToRgb( colorValue ),
                alpha = asAlpha( transparencyValue );

            return "rgba(" + color.r + ", " + color.g + ", " + color.b + ", " + alpha + ")";
        }

        function asAlpha( percentValue ) {
            return (100 - percentValue) / 100;
        }

        // PLAYMODES CREATION FUNCTIONS
        function createPlaymodes()  {

            var imageOptions = options.image.button;

            playmodesButton = jq(document.createElement("div"))
                .attr({
                    "id": "playmodesButton" + options.instanceID,
                    "title" : options.caption
                })
                .css({
                    "width": (imageOptions.width / 3) + "px",
                    "height": (imageOptions.height / 3) + "px"
                })
                .addClass("playmodesButton");

            playmodesButtonHotSpotElement = jq(document.createElement("div"))
                .attr({
                    "id": "playmodesButtonHotSpotElement" + options.instanceID,
                    "title" : options.caption
                })
                .css({
                    "width": (imageOptions.width / 3) + "px",
                    "height": (imageOptions.height / 3) + "px"
                })
                .addClass("playmodesButtonHotSpotElement");

            button = new ToggleButtonBuilder({
                fileNameURL : imageOptions.fileNameURL,
                width : imageOptions.width / 3,
                height : imageOptions.height / 3,
                name : "playmodesButton",
                captionText : options.caption}, context, metadata).build();

            playmodesContainer = createPlaymodesContainer();

            playmodesButton.append(button.getElement());
            playmodesButton.append(playmodesButtonHotSpotElement);
            playmodesButton.append(playmodesContainer.getElement());
        }

        function createPlaymodesContainer() {

            var items = new Array(),
                isContainerOpen = false,
                position = getMenuPosition(),
                fillColor = options.fillColor,
                fillTransparency = options.fillTransparency,
                borderColor = options.borderColor,
                borderTransparency = options.borderTransparency,
                container = jq(document.createElement("div"))
                    .attr({
                        "id": "playmodesContainer" + options.instanceID
                    })
                    .css({
                        "width": options.menuWidth + "px",
                        "height": options.menuHeight + "px",
                        "border": options.borderWidth + "px solid " + asRgbaValue(borderColor, borderTransparency),
                        "background-color" : asRgbaValue(fillColor, fillTransparency)
                    })
                    .css(position)
                    .addClass("playmodesContainer")
                    .on(options.pointerEvents.pointerDown, function(e){
                        cancelEvent(e);
                        lockNavigation();
                    })
                    .on(options.pointerEvents.pointerUp, function(e){
                        cancelEvent(e);
                        unlockNavigation();
                    })
                    .on("dblclick", cancelEvent );

                context.page.interaction.provokesFeedback( container, false );
                context.page.interaction.provokesAutosolve( container, false );

            if(options.AuthoringMode && options.showMenuInAuthoringMode){
                container.css({
                    "opacity" : "0.15",
                    "background-image" : "url(\"" + context.getPath() + "invis.png\")",
                    "text-align" : "center",
                    "line-height" : options.menuHeight + "px",
                    "font-size" : "24px"
                })
                .text( "Play modes" )
                .show();
            }else{
                container.hide();
            }

            return {
                getElement : function(){
                    return container.get(0);
                },
                addItem : function(item){             
                    items.push(item);
                    container.append(item.getElement());
                },
                getItems : function(){
                    return items;
                },
                hideContainer : function(){
                    isContainerOpen = false;
                    container.hide();
                },
                showContainer : function(){
                    isContainerOpen = true;
                    container.show();
                },
                isOpen : function(){
                    return isContainerOpen;
                },
                resetPlaymodesContainer : function(){
                    items.length = 0;
                    container.empty();
                }
            }
        }

        function createPlaymodesContent( menuContainer ) {

            var availablePlayModes = metadata.stage.getAvailablePlayModes(),
                individualPlayMode = getIndividualPlayMode();

            if ((individualPlayMode === "none" || individualPlayMode === "interactive") && availablePlayModes.interactive) {
                menuContainer.addItem(createMenuItem( "interactive", options.entryTexts.interactive, function(e){
                    cancelEvent(e);
                    unlockNavigation();
                    context.stage.navigation.switchToInteractiveMode();
                    menuContainer.hideContainer();
                    setMarker( "interactive" );
                    button.changeToggleState("btn_state_interactive");
                }));
            }

            if ((individualPlayMode === "none" || individualPlayMode === "presentation") && availablePlayModes.presentation) {
                menuContainer.addItem(createMenuItem("presentation", options.entryTexts.presentation, function(e){
                    cancelEvent(e);
                    unlockNavigation();
                    context.stage.navigation.switchToPresentationMode();
                    menuContainer.hideContainer();
                    setMarker( "presentation" );
                    button.changeToggleState("btn_state_presentation");
                }));
            }

            if ((individualPlayMode === "none" || individualPlayMode === "film") && availablePlayModes.film) {
                menuContainer.addItem(createMenuItem("film", options.entryTexts.film, function(e){
                    cancelEvent(e);
                    unlockNavigation();
                    context.stage.navigation.switchToFilmMode();
                    menuContainer.hideContainer();
                    setMarker( "film" );
                    button.changeToggleState("btn_state_film");
                }));
            }
        }

         function getIndividualPlayMode() {

             if( metadata.page && metadata.page.getIndividualPlayMode ){
                 return metadata.page.getIndividualPlayMode();
             }

             return "none";
         }

         function createMenuItem( id, caption, action ) {

            var item,
                marker,
                elementId = id,
                textColor = options.textColor,
                hoverColor = options.textHoverColor,
                separatorColor = options.separatorColor,
                separatorTransparency = options.separatorTransparency,
                itemPaddingLeft = parseInt(options.paddingLeft),
                itemPaddingRight = parseInt(options.paddingRight) + parseInt(options.image.marker.width) + 10,
                itemWidth = parseInt(options.menuWidth) - itemPaddingLeft - itemPaddingRight;

            item = jq(document.createElement("div"))
            .attr( "id", id + "-item" + options.instanceID )
            .css({
                "border-bottom": options.separatorWidth + "px solid " + asRgbaValue(separatorColor, separatorTransparency),
                "color": textColor,
                "font-family": options.fontFamily,
                "padding-left": itemPaddingLeft + "px",
                "padding-right": itemPaddingRight  + "px",
                "padding-top": options.paddingTop + "px",
                "padding-bottom": options.paddingBottom + "px",
                "font-size": options.fontSize + "px",
                "width" : itemWidth + "px",
            })
            .addClass("item text")
            .text(caption)
            .on(options.pointerEvents.pointerDown, function(e){
                cancelEvent(e);
                lockNavigation();
            })
            .on(options.pointerEvents.pointerUp, action)
            .on(options.pointerEvents.pointerMove, function(){
                jq(this).css("color", hoverColor);
            })
            .on(options.pointerEvents.pointerLeave, function(){
                jq(this).css("color", textColor);
            })
            .on("dblclick", cancelEvent );

            if(id == "film"){
                item.css({
                    "border-bottom": "none"
                })
            }

            marker = jq(document.createElement("img"))
                .attr({
                    "id" : id + "-marker" + options.instanceID,
                    "src" : options.image.marker.fileNameURL
                })
                .css({
                    "width": options.image.marker.width + "px",
                    "height": options.image.marker.height + "px"
                })
                .addClass("marker")
                .hide();

            item.append( marker );

            return {
                getId : function(){
                    return elementId;
                },
                getElement : function(){
                    return item;
                },
                setMarked : function(){
                    marker.show();
                },
                unsetMarked : function(){
                    marker.hide();
                }
            }
        }

        // PLAYMODES HELPER FUNCTIONS

        function getMenuPosition(){

            var buttonWidth = options.image.button.width / 3,
                buttonHeight = options.image.button.height / 3,
                menuWidth = options.menuWidth,
                menuHeight = options.menuHeight,
                border = options.borderWidth,
                menuPositions = [
                    //bottom-left
                    {
                        "top": buttonHeight + "px",
                        "right": menuWidth + (border * 2) + "px"
                    },
                    //top-left
                    {
                        "bottom": menuHeight + (border * 2) + "px",
                        "right": menuWidth + (border * 2) + "px"
                    },
                    //top-centered
                    {
                        "bottom": menuHeight + (border * 2) + "px",
                        "right": (menuWidth + (border * 2) - buttonWidth) / 2 + "px"
                    },
                    //top-right
                    {
                        "bottom": menuHeight + (border * 2) + "px",
                        "left": buttonWidth + "px"
                    },
                    //right-centered
                    {
                        "bottom": (menuHeight + (border * 2) - buttonWidth) / 2 + "px",
                        "left": buttonWidth + "px"
                    },
                    //bottom-right
                    {
                        "top": buttonHeight + "px",
                        "left": buttonWidth + "px"
                    },
                    //bottom-centered
                    {
                        "top": buttonHeight + "px",
                        "right": (menuWidth + (border * 2) - buttonWidth) / 2 + "px"
                    },
                    //left-centered
                    {
                        "bottom": (menuHeight + (border * 2) - buttonWidth) / 2 + "px",
                        "right": menuWidth + (border * 2) + "px"
                    }
                ],
                posOption = options.position;

            if(posOption != 8){
                return menuPositions[ posOption ];
            }else{
                return {
                    "top": options.menuPositionY + "px",
                    "left": options.menuPositionX + "px"
                };
            }
        }

        function setMarker( id ){
            var items = playmodesContainer.getItems();
            jq.each( items, function(index,item){
                item.unsetMarked();
                if( item.getId() === id ){
                    item.setMarked();
                }
            });
        }

        // EVENT HELPER FUNTIONS
        function lockNavigation() {
            if (context.stage.isInAuthoringMode() || (typeof(context.stage.trigger) === "undefined")) {
                return;
            }
            context.stage.trigger.lock( metadata.getId() );
        }

        function unlockNavigation() {
            if (context.stage.isInAuthoringMode() || (typeof(context.stage.trigger) === "undefined")) {
                return;
            }
            context.stage.trigger.unlock( metadata.getId() );
        }

        function isOnTarget(e) {
            var elem = e.target,
                origEvent = e.originalEvent,
                changedTouch = origEvent.changedTouches && origEvent.changedTouches.length > 0 ? origEvent.changedTouches[0] : undefined;

            if (changedTouch) {
                elem = document.elementFromPoint(changedTouch.clientX, changedTouch.clientY);
            }

            return e.target === elem;
        }

        function cancelEvent( e ){
            e.preventDefault();
            e.stopPropagation();
        }

        return {
            build : function(){
                createPlaymodes();
                return {
                    getElement : function(){
                        return playmodesButton.get(0);
                    },
                    getHotSpotElement : function(){
                        return playmodesButtonHotSpotElement.get(0);
                    },
                    onMouseLeave : function(){
                        button.onMouseLeave();
                    },
                    onMouseOver : function(){
                        button.onMouseOver();
                    },
                    onMouseDown : function(e){
                        lockNavigation();
                        e.preventDefault();
                        e.stopPropagation();

                        button.onMouseDown(e);
                    },
                    onMouseUp : function(e){
                        unlockNavigation();
                        e.preventDefault();
                        e.stopPropagation();

                        button.onMouseLeave();
                        
                        if(!isOnTarget(e)) {
                            return;
                        }

                        if(playmodesContainer.isOpen()){
                            playmodesContainer.hideContainer();
                        }else{
                            playmodesContainer.showContainer();
                        }

                        button.onMouseUp(e);
                    },
                    onEnterPage : function() {
                        createPlaymodesContent(playmodesContainer);         
                    },
                    onExitPage : function() {
                        playmodesContainer.hideContainer();
                        playmodesContainer.resetPlaymodesContainer();
                        button.onMouseLeave();
                    },

                    setInteractiveMode : function() {
                        setMarker( "interactive" );
                        button.changeToggleState("btn_state_interactive");
                    },
                    setPresentationMode : function() {
                        setMarker( "presentation" );
                        button.changeToggleState("btn_state_presentation");
                    },
                    setFilmMode : function() {
                        setMarker( "film" );
                        button.changeToggleState("btn_state_film");
                    }
                };
            }
        }
    };
});
